# Generated by roxygen2: do not edit by hand

export(FORM)
export(MC_CRUDE)
export(MC_IS)
export(MC_SubSam)
export(MVFOSM)
export(PARAM_BASEVAR)
export(PARAM_DETVAR)
export(PARAM_LSF)
export(PROB_BASEVAR)
export(PROB_DETVAR)
export(PROB_MACHINE)
export(SORM)
export(SYS_LSF)
export(SYS_PARAM)
export(SYS_PROB)
export(dlt)
export(dst)
export(plt)
export(pst)
export(qlt)
export(qst)
export(rlt)
export(rst)
exportClasses(PARAM_BASEVAR)
exportClasses(PARAM_DETVAR)
exportClasses(PARAM_LSF)
exportClasses(PROB_BASEVAR)
exportClasses(PROB_DETVAR)
exportClasses(PROB_MACHINE)
exportClasses(SYS_LSF)
exportClasses(SYS_PARAM)
exportClasses(SYS_PROB)
import(digest)
import(future)
import(future.apply)
import(ggplot2)
import(gridExtra)
import(nloptr)
import(parallel)
import(pracma)
importFrom(future,availableCores)
importFrom(future,multicore)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(methods,"new")
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,clusterSetRNGStream)
importFrom(parallel,makeCluster)
importFrom(stats,dnorm)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rnorm)
importFrom(utils,str)
