% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallel_helpers.R
\name{create_rng_manager}
\alias{create_rng_manager}
\title{Deterministic RNG Block Manager (L'Ecuyer-CMRG)}
\usage{
create_rng_manager(seed = NULL)
}
\arguments{
\item{seed}{Optional integer for reproducibility.}
}
\value{
An RNG manager object (environment).
}
\description{
Provides deterministic generation of random number blocks that are
independent of the number of worker threads.

The RNG state is advanced only by the requested block size.
}
\keyword{internal}
