% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallel_helpers.R
\name{make_parallel_cluster}
\alias{make_parallel_cluster}
\title{Create a parallel cluster (fork on *nix, PSOCK on Windows)}
\usage{
make_parallel_cluster(use_threads, seed = NULL)
}
\arguments{
\item{use_threads}{Integer \>=1 – number of workers.}

\item{seed}{Optional integer or L'Ecuyer‑CMRG stream (length 7) for reproducibility.}
}
\value{
A cluster object (or NULL when the backend is "future").
}
\description{
Returns a `parallel` cluster object that already has the
  RNG streams (L'Ecuyer‑CMRG) and required packages loaded.
}
\keyword{internal}
