% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MC_SubSam.R
\name{MC_SubSam}
\alias{MC_SubSam}
\title{Monte Carlo Simulation with Subset-Sampling}
\usage{
MC_SubSam(
  lsf,
  lDistr,
  Nsubset = 1e+05,
  p0 = 0.1,
  MaxSubsets = 10,
  Alpha = 0.05,
  variance = "uniform",
  debug.level = 0,
  seed = NULL
)
}
\arguments{
\item{lsf}{limit-state function}

\item{lDistr}{list of basevariables in input space}

\item{Nsubset}{number of samples in each simulation level}

\item{p0}{level probability or conditional probability}

\item{MaxSubsets}{maximum number of simulation levels that are used to terminate the simulation procedure to avoid infinite loop when the target domain cannot be reached}

\item{Alpha}{confidence level}

\item{variance}{gaussian, uniform}

\item{debug.level}{If 0 no additional info if 2 high output during calculation}

\item{seed}{Optional integer value. If specified, sets a fixed seed for reproducible random numbers (useful for testing).}
}
\value{
MC_SubSam returns an object containing the following elements:
\itemize{
  \item \code{beta}: Estimated reliability index.
  \item \code{pf}: Estimated probability of failure.
  \item \code{betaCI}: Confidence interval of the reliability index.
  \item \code{pfCI}: Confidence interval of the probability of failure.
  \item \code{CoV}: Coefficient of variation of the estimator.
  \item \code{NumOfSubsets}: Number of Markov chains (subsets).
  \item \code{NumOfEvalLSF_nom}: Nominal number of limit-state function evaluations
        (Markov chains / iterations).
  \item \code{NumOfEvalLSF_eff}: Effective number of limit-state function evaluations.
  \item \code{runtime}: Total runtime of the function.
}
}
\description{
MonteCarlo with Subset-Sampling (MC_SubSam) method for the
estimation of small failure probabilities in structural reliability analysis.

Subset Simulation is an advanced Monte Carlo technique that
expresses a rare failure event as a sequence of intermediate
conditional events with higher probabilities.

The method combines conditional sampling with Markov Chain Monte Carlo (MCMC)
techniques to efficiently estimate very small probabilities of failure,
which would be computationally expensive to obtain using crude Monte Carlo simulation.
}
\references{
Au, S. K., & Beck, J. L. (2001).
Estimation of small failure probabilities in high dimensions
by subset simulation.
\emph{Probabilistic Engineering Mechanics}, 16(4), 263-277.

Marelli, S., & Sudret, B. (2014).
UQLab: A framework for uncertainty quantification in Matlab.
In \emph{Proceedings of the 2nd International Conference on
Vulnerability, Risk Analysis and Management (ICVRAM2014)},
Liverpool, United Kingdom, 2554-2563.
}
\author{
(C) 2021-2026 K. Nille-Hauf, T. Feiri, M. Ricker, T. Lux -- Hochschule Biberach (until 2022),
TU Dortmund University - Chair of Structural Concrete (since 2023)
}
