% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TesiproV.R
\docType{package}
\name{TesiproV-package}
\alias{TesiproV}
\alias{TesiproV-package}
\title{TesiproV: A package for the calculation of reliability and failure probability in civil engineering}
\description{
The Package provides three main types of objects:
\enumerate{
\item Objects for modeling base variables
\item Objects for modeling limit state functions and systems of them
\item Objects for modeling solving algorithms
}
}
\details{
By creating and combining those objects, one is able to model quite complex problems in terms of structural reliablity calculation.
For normally distributed variables there might be an workflow to calculate correlated problems (but no systems then).
There is also implemented a new distribution (logStudentT, often used for conrete compression strength) to show how one can implement
your very own or maybe combined multi modal distribution and use it with TesiproV.
}
\section{Objects for base variables}{

\code{PROB_BASEVAR}, \code{PROB_DETVAR}, \code{PARAM_BASEVAR}, \code{PARAM_DETVAR}
}

\section{Limit state functions}{

\code{SYS_LSF}, \code{PROB_SYS}, \code{PARAM_SYS}
}

\section{Solving algorithms}{

\code{PROB_MACHINE}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://mvb.ab.tu-dortmund.de/}
}

}
\author{
(C) 2021-2026 K. Nille-Hauf, T. Feiri, M. Ricker, T. Lux -- Hochschule Biberach (until 2022),
TU Dortmund University - Chair of Structural Concrete (since 2023)
}
